#! /usr/bin/perl -T -w
#-*-Perl-*-
#
# Copyright (c) 2004 Lucio Henrique Franco (lucio@lac.inpe.br) and
#          Carlos Henrique Peixoto Caetano Chaves (cae@lac.inpe.br)
#
# All rights reserved.
#            Renato Archer Research Center (CenPRA)
#            Brazilian National Institute for Space Research (INPE)
#            Information System and Network Security Group
#            version 0.2 - Tue Mar  9 16:05:19 BRT 2004
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
#    - Redistributions of source code must retain the above copyright
#      notice, this list of conditions and the following disclaimer.
#    - Redistributions in binary form must reproduce the above
#      copyright notice, this list of conditions and the following
#      disclaimer in the documentation and/or other materials 
#      provided with the distribution.
#    - All advertising materials mentioning features or use of this
#      software must display the following acknowledgement:
#      This product includes software developed by Lucio Henrique
#      Franco and Carlos H. P. C. Chaves. CenPRA and INPE aren't 
#      responsible by use or distribution of this material.
#    - Neither the name of the Research Center nor the names of its
#      contributors may be used to endorse or promote products
#      derived from this software without specific prior written 
#      permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#

##########################################################################
### Modules used by this script

use strict;
use Getopt::Std;

our $program_name;
($program_name = $0) =~ s@.*/@@;

unless (eval "use Net::Netmask; 1") {
    die "$program_name: please install Net::Netmask.";
}

##########################################################################
### Program name, version and options

our %option = ();

getopts('c:i:hn:p:s:t:V', \%option) || die "$program_name: cannot 
                                                get options.\n";

our $honeydsum_version = '0.2';

##########################################################################
### Some global declarations

# set PATH for this script
$ENV{'PATH'} = '/bin:/usr/bin:/usr/local/bin';

# unbuffered output
$| = 1;

# IP regexp
our $IP_exp = '\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}';

# Timestmap regexp
our $timestamp_exp = '\d{4}\-\d{2}\-\d{2}\-\d{2}\:\d{2}\:\d{2}\.\d{4}';

# Time regexp
our $time = '\d+:\d+:\d+';

# used to read files from command line
our $cat   = '/bin/cat';
our $zcat  = '/usr/bin/zcat';
our $bzcat = '/usr/local/bin/bzcat';

# honeyd.conf file
our $honeyd_conf_file;

# used to show icmp protocol
our ($proto_show) = 0;

# used to show the number of elements of top
our ($top_show) = 11;

##########################################################################
### Checking arguments

# display version if requested
if (defined($option{V})) {
   &show_version;
}

# check if there are arguments
if ($#ARGV == -1 || defined($option{h})) {
   &show_usage;
}

##########################################################################
### Local variables for Main

my ($file, $viewer, $line);
my ($totalpkts, $tcppkts, $udppkts, $icmppkts);
my (
   $timestamp, $proto, $src_ip, $dst_ip,     $resource,
   $port,      $hour,  $bytes,  $nbr_src_ip, $nbr_dst_ip
);
my (@IP_list)    = ();
my (@NET_list)   = ();
my (@PORT_list)  = ();
my (@PROTO_list) = ();

my (%stat_hash)     = ();
my (%src_host_hash) = ();
my (%resource_hash) = ();
my (%hour_hash)     = ();
my (%icmp_b40_hash) = ();

my ($cnt, $dez, $uni);
##########################################################################
###  Main

if (defined($option{'c'})) {
   $honeyd_conf_file = $option{'c'};
}

if (defined($option{'i'})) {
   @IP_list = &set_list("IP", $option{'i'});

   if ($#IP_list == -1) {
      warn("$program_name: option '-i': cannot set IP list.\n");
   }
}

if (defined($option{'n'})) {
   @NET_list = &set_list("NET", $option{'n'});

   if ($#NET_list == -1) {
      warn(
         "$program_name: option '-n': cannot set IP or net
	      list.\n"
      );
   }
}

if (defined($option{'p'})) {
   @PORT_list = &set_list("PORT", $option{'p'});

   if ($#PORT_list == -1) {
      warn("$program_name: option '-p': cannot set port list.\n");
   }
}

if (defined($option{'t'})) {
   @PROTO_list = &set_list("PROTO", $option{'t'});

   if ($#PROTO_list == -1) {
      warn("$program_name: option '-t': cannot set protocol
	      list.\n");
   }
}

if (defined($option{'s'})) {
   if ($option{'s'} =~ /([0-9]+)/) {
      $top_show = $1 + 1;
   } else {
      warn("$program_name: option '-s': cannot set top.\n");
   }
}

if (defined($option{'-'})) {
   push(@ARGV, "-");
}

# Erase hour hash
$dez = 48;
$uni = 48;
while (!($dez == 50 && $uni == 52)) {
   $hour_hash{ chr($dez) . chr($uni) } = 0;
   if ($uni == 57) {
      $dez++;
      $uni = 48;
   } else {
      $uni++;
   }
}

$totalpkts = 0;
$tcppkts   = 0;
$udppkts   = 0;
$icmppkts  = 0;
foreach my $arg (@ARGV) {
   if (!($file = &check_filename($arg))) {
      $file = quotemeta($arg);
      warn("$program_name: %s: invalid file name.\n", $file);
      next;
   }

   if ($file =~ /\.bz2$/) {
      $viewer = $bzcat;
   } elsif ($file =~ /\.gz$/) {
      $viewer = $zcat;
   } else {
      $viewer = $cat;
   }

   my @viewer_args = ();
   push(@viewer_args, $file);
   my $pid = open(CHILD_TO_READ1, "-|");

   if (!$pid) {

      # child
      exec($viewer, @viewer_args)
        || die("$program_name: $viewer: cannot exec: $!\n");

      # never reached
   } else {

      # parent
      my $line_count = 0;

      while ($line = <CHILD_TO_READ1>) {
         $line_count++;
         chomp($line);

         ##############
         #2003-12-16-16:50:29.0837 tcp(6) - xxx.xxx.xxx.xxx 37604
         #yyy.yyy.yyy.yyy 23: 60 S
         #2003-12-16-16:50:29.0863 tcp(6) - xxx.xxx.xxx.xxx 37608
         #yyy.yyy.yyy.yyy 23: 60 S [Linux 2.6 ]
         #2003-12-16-16:50:32.0824 tcp(6) S xxx.xxx.xxx.xxx 37614
         #yyy.yyy.yyy.yyy 23 [Linux 2.6 ]
         #2003-12-16-16:50:33.0388 tcp(6) E xxx.xxx.xxx.xxx 37614
         #yyy.yyy.yyy.yyy 23: 0 0
         #2003-12-16-16:50:41.0267 tcp(6) S xxx.xxx.xxx.xxx 1938
         #yyy.yyy.yyy.yyy 8080 [Windows 2000 SP4]
         #2003-12-16-16:50:42.0484 tcp(6) E xxx.xxx.xxx.xxx 1940
         #yyy.yyy.yyy.yyy 6588: 151 0
         ##############

         if ($line =~
		 /^($timestamp_exp)\s+(tcp)\(6\)\s+[S|-]\s($IP_exp)\s+\d{1,5}\s+($IP_exp)\s+(\d{1,5})+.*/)
	 {

            $timestamp = $1;
            $proto     = $2;
            $src_ip    = $3;
            $dst_ip    = $4;
            $resource  = $5;

            if (
               (($#IP_list == -1) || (&check_list($dst_ip, @IP_list)))
               && (  ($#NET_list == -1)
                  || (&check_net_list($src_ip)))
               && (($#PORT_list == -1) || (&check_list($resource, @PORT_list)))
               && (($#PROTO_list == -1) || (&check_list($proto, @PROTO_list)))
              )
            {

               $totalpkts++;
               $tcppkts++;

               $nbr_src_ip = unpack "N", pack "C4", split /\./, $src_ip;
               $nbr_dst_ip = unpack "N", pack "C4", split /\./, $dst_ip;

               if (
                  !defined(
                     $stat_hash{$nbr_dst_ip}{$nbr_src_ip}{$proto}{$resource}
                  )
                 )
               {
                  $stat_hash{$nbr_dst_ip}{$nbr_src_ip}{$proto}{$resource} = 0;
               }
               $stat_hash{$nbr_dst_ip}{$nbr_src_ip}{$proto}{$resource}++;

               if (!defined($src_host_hash{$src_ip})) {
                  $src_host_hash{$src_ip} = 0;
               }
               $src_host_hash{$src_ip}++;

               if (!defined($resource_hash{ $resource . "/" . $proto })) {
                  $resource_hash{ $resource . "/" . $proto } = 0;
               }
               $resource_hash{ $resource . "/" . $proto }++;

               if ($timestamp =~ /^\d{4}\-\d{2}\-\d{2}\-(\d{2})\:.*$/) {
                  $hour = $1;
                  $hour_hash{$hour}++;
               }
            }
         } elsif ($line =~
		 /^($timestamp_exp)\s+(udp)\(17\)\s+[S|-]\s+($IP_exp)\s+\d{1,5}\s+($IP_exp)\s+(\d{1,5}).*/)
	 {
             #2003-10-03-21:46:40.0864 udp(17) - xxx.xxx.xxx.xxx 138 yyy.yyy.yyy.yyy 138: 229
             #2003-12-17-19:41:19.002 udp(17) S xxx.xxx.xxx.xxx 53 yyy.yyy.yyy.yyy 53
             #2003-12-17-19:42:19.017 udp(17) E xxx.xxx.xxx.xxx 53 yyy.yyy.yyy.yyy 53: 18 0

            $timestamp = $1;
            $proto     = $2;
            $src_ip = $3;
            $dst_ip   = $4;
            $resource = $5;

            if (  (($#IP_list == -1) || (&check_list($dst_ip, @IP_list)))
               && (($#NET_list == -1)   || (&check_net_list($src_ip)))
               && (($#PORT_list == -1)  || (&check_list($resource, @PORT_list)))
               && (($#PROTO_list == -1) || (&check_list($proto, @PROTO_list))))
            {
               $totalpkts++;
               $udppkts++;

               $nbr_src_ip = unpack "N", pack "C4", split /\./, $src_ip;
               $nbr_dst_ip = unpack "N", pack "C4", split /\./, $dst_ip;

               if (
                  !defined(
                     $stat_hash{$nbr_dst_ip}{$nbr_src_ip}{$proto}{$resource}
                  )
                 )
               {
                  $stat_hash{$nbr_dst_ip}{$nbr_src_ip}{$proto}{$resource} = 0;
               }
               $stat_hash{$nbr_dst_ip}{$nbr_src_ip}{$proto}{$resource}++;

               if (!defined($src_host_hash{$src_ip})) {
                  $src_host_hash{$src_ip} = 0;
               }
               $src_host_hash{$src_ip}++;

               if (!defined($resource_hash{ $resource . "/" . $proto })) {
                  $resource_hash{ $resource . "/" . $proto } = 0;
               }
               $resource_hash{ $resource . "/" . $proto }++;

               if ($timestamp =~ /^\d{4}\-\d{2}\-\d{2}\-(\d{2})\:.*$/) {
                  $hour = $1;
                  $hour_hash{$hour}++;
               }
            } 
         } elsif ($line =~
		 /^($timestamp_exp)\s+(icmp)\(1\)\s+\-\s+($IP_exp)\s+($IP_exp)\:\s+(\d{1,2})\(\d{1,2}\)\:\s+(\d+)/)
	 {
            #2003-09-13-00:00:15.0775 icmp(1) - xxx.xxx.xxx.xxx yyy.yyy.yyy.yyy: 8(0): 92
            $timestamp = $1;
            $proto     = $2;
            $src_ip    = $3;
            $dst_ip    = $4;
            $resource  = $5;
            $bytes     = $6;

            if (  (($#IP_list == -1) || (&check_list($dst_ip, @IP_list)))
               && (($#NET_list == -1)   || (&check_net_list($src_ip)))
               && (($#PORT_list == -1)  || (&check_list($resource, @PORT_list)))
               && (($#PROTO_list == -1) || (&check_list($proto, @PROTO_list))))
            {
               $totalpkts++;
               $icmppkts++;

               $nbr_src_ip = unpack "N", pack "C4", split /\./, $src_ip;
               $nbr_dst_ip = unpack "N", pack "C4", split /\./, $dst_ip;

               if (
                  !defined(
                     $stat_hash{$nbr_dst_ip}{$nbr_src_ip}{$proto}{$resource}
                  )
                 )
               {
                  $stat_hash{$nbr_dst_ip}{$nbr_src_ip}{$proto}{$resource} = 0;
               }
               $stat_hash{$nbr_dst_ip}{$nbr_src_ip}{$proto}{$resource}++;

               if (!defined($src_host_hash{$src_ip})) {
                  $src_host_hash{$src_ip} = 0;
               }
               $src_host_hash{$src_ip}++;

               if (!defined($resource_hash{ $resource . "/" . $proto })) {
                  $resource_hash{ $resource . "/" . $proto } = 0;
               }
               $resource_hash{ $resource . "/" . $proto }++;

               if ($timestamp =~ /^\d{4}\-\d{2}\-\d{2}\-(\d{2})\:.*$/) {
                  $hour = $1;
                  $hour_hash{$hour}++;
               }

               if ($bytes > 40) {
                  if (!defined($icmp_b40_hash{$src_ip})) {
                     $icmp_b40_hash{$src_ip} = 0;
                  }
                  $icmp_b40_hash{$src_ip}++;
               }
            }
         } else {
            warn("$program_name: unknown format: $line\n")
         }
      }    #end while
      close(CHILD_TO_READ1) || warn("$program_name: $viewer: exited $?\n");
   }    #end else
}    #end foreach ARGV

if (defined($option{'c'})) {
   &ext_honeyd_conf;
}

printf("\n--------------------------------------\n");
printf("Connection Counter\n");
printf("--------------------------------------\n");
printf("Total: %10d\n", $totalpkts);
printf("  TCP: %10d\n", $tcppkts);
printf("  UDP: %10d\n", $udppkts);
printf(" ICMP: %10d\n", $icmppkts);
printf("--------------------------------------\n\n");

foreach $nbr_dst_ip (sort { $a <=> $b } keys %stat_hash) {
   my %src_ip_hash = %{ $stat_hash{$nbr_dst_ip} };
   my $ip_flag;
   my ($total_connections)     = 0;
   my (@total_resource)        = ();
   my ($total_resource_number) = 0;
   my ($total_ip)              = 0;

   $dst_ip = join ".", unpack "C4", pack "N", $nbr_dst_ip;

   printf("--------------------------------------\n");
   printf("Honeypot: %s\n", $dst_ip);
   printf("--------------------------------------\n");
   printf("%-15s %9s %12s\n", "Source IP", "Resource", "Connections");

   foreach $nbr_src_ip (sort { $a <=> $b } keys %src_ip_hash) {
      $ip_flag = 0;

      my %proto_hash = %{ $src_ip_hash{$nbr_src_ip} };

      $src_ip = join ".", unpack "C4", pack "N", $nbr_src_ip;

      foreach $proto (keys %proto_hash) {
         my %resource_hash = %{ $proto_hash{$proto} };

         foreach $resource (sort { $a <=> $b } keys %resource_hash) {
            if ($ip_flag == 0) {
               printf("%-15s %5s/%-5s %5d\n",
                  $src_ip, $resource, $proto, $resource_hash{$resource});
               $ip_flag      = 1;
               $total_connections = $total_connections + $resource_hash{$resource};
               $total_ip++;
               push(@total_resource, $resource);
            } else {
               printf("%-15s %5s/%-5s %5d\n",
                  " ", $resource, $proto, $resource_hash{$resource});
            }
         }
      }
   }

   my (%seen) = ();
   my (@uniq) = ();
   my ($item) = ();
   foreach $item (@total_resource) {
      $total_resource_number++ unless $seen{$item}++;
   }

   printf("--------------------------------------\n");
   printf("%-15s %9s %12s\n", "IPs", "Resources", "Connections");
   printf("%-15s %9s %8s\n", $total_ip, $total_resource_number, $total_connections);
   printf("--------------------------------------\n");
}

printf("\n\nTop %s Source Hosts\n\n", ($top_show - 1));
printf("%-4s %-15s %9s\n", "Rank", "Source IP", "Connections");

$cnt = 1;
foreach $src_ip (sort { $src_host_hash{$b} <=> $src_host_hash{$a} }
   keys %src_host_hash)
{
   printf("%-4d %-15s %8d\n", $cnt, $src_ip, $src_host_hash{$src_ip});
   $cnt++;
   if ($cnt == $top_show) {
      last;
   }
}

printf("\n\nTop %s Accessed Resources\n\n", ($top_show - 1));
printf("%-4s %-11s %9s\n", "Rank", "Resource", "Connections");

$cnt = 1;
foreach $port (sort { $resource_hash{$b} <=> $resource_hash{$a} }
   keys %resource_hash)
{
   printf("%-4d %-11s %8d\n", $cnt, $port, $resource_hash{$port});
   $cnt++;
   if ($cnt == $top_show) {
      last;
   }
}

if ($proto_show == 0 || $proto_show == 5 || $proto_show == 6) {
   printf("\n\nTop %s ICMP > 40 bytes Senders\n\n", ($top_show - 1));
   printf("%-4s %-15s %9s\n", "Rank", "Source IP", "Connections");

   $cnt = 1;
   foreach $src_ip (sort { $icmp_b40_hash{$b} <=> $icmp_b40_hash{$a} }
      keys %icmp_b40_hash)
   {
      printf("%-4d %-15s %8d\n", $cnt, $src_ip, $icmp_b40_hash{$src_ip});
      $cnt++;
      if ($cnt == $top_show) {
         last;
      }
   }
}

printf("\n\nConnections per Hour\n\n");
printf("%-5s %9s\n", "Hour", "Connections");

foreach $hour (sort { $a cmp $b } keys %hour_hash) {
   printf("%s:00 %8d\n", $hour, $hour_hash{$hour});
}

exit 0;

##########################################################################
### Subroutines

#-------------------------------------------------------------------------
# Name: ext_honeyd_conf
#
# Description: Extracts the configuration of honeyd
#

sub ext_honeyd_conf {
   my ($file, $viewer, $line);
   my ($pid);
   my ($control);
   my ($create_flag);
   my ($personality);
   my ($system_aux) = ();
   my (@tcp_ports)  = ();
   my (@udp_ports)  = ();
   my (@ips)        = ();
   my (@ips_aux)    = ();

   if (!($file = &check_filename("./" . $honeyd_conf_file))) {
      $file = quotemeta("./" . $honeyd_conf_file);
      warn("$program_name: $file: invalid file name.\n");
      next;
   }

   if ($file =~ /\.bz2$/) {
      $viewer = $bzcat;
   } elsif ($file =~ /\.gz$/) {
      $viewer = $zcat;
   } else {
      $viewer = $cat;
   }

   my @viewer_args = ();
   push(@viewer_args, $file);

   printf("### Honeypot's Configuration ###\n\n");

   $pid = open(CHILD_TO_READ3, "-|");

   if (!$pid) {

      # child
      exec($viewer, @viewer_args)
        || die("$program_name: $viewer: cannot exec: $!\n");

      # never reached
   } else {

      # parent

      $create_flag = 1;

      while ($line = <CHILD_TO_READ3>) {
         if ($create_flag < 0) {

            if (defined($option{'i'})) {
               $control = 0;
               if ($#ips != -1) {
                  foreach my $ip (@ips) {
                     if (&check_list($ip, @IP_list) == 1) {
                        $control = 1;
                        push(@ips_aux, $ip);
                     }
                  }
               }
            } else {
               $control = 1;
            }
            if ($control == 1) {    #it found the ip
               printf("\n");
               printf($personality);
               printf("\n");

               if (defined($option{'i'})) {
                  @ips = @ips_aux;
               }
               if ($#ips != -1) {
                  printf("\t( / ");
                  foreach my $ip (@ips) {
                     printf("%s / ", $ip);
                  }
                  printf(")\n");
               }

               if ($#tcp_ports != -1) {
                  foreach my $port (@tcp_ports) {
                     printf("\t%s/tcp\n", $port);
                  }
               }

               if ($#udp_ports != -1) {
                  foreach my $port (@udp_ports) {
                     printf("\t%s/udp\n", $port);
                  }
               }

            }
            @tcp_ports = ();
            @udp_ports = ();
            @ips       = ();
            @ips_aux   = ();

            $create_flag++;
         }

         if ($line =~ /^create\s+(.*)/) {
            $create_flag--;
            $system_aux = $1;
         } elsif ($line =~ /^\w.*personality\s+\"(.*)\"$/) {
            $personality = $1;
         } elsif ($line =~ /^\w+(.*)\s+tcp port\s+(\d{1,5})\s+.*/) {
            push(@tcp_ports, $2);
         } elsif ($line =~ /^\w+(.*)\s+udp port\s+(\d{1,5})\s+.*/) {
            push(@udp_ports, $2);
         } elsif ($line =~ /^bind\s+($IP_exp)\s+(.*)/) {
            push(@ips, $1);
         }
      }

   }

   close(CHILD_TO_READ3) || warn("$program_name: $viewer: exited $?\n");
}

#-------------------------------------------------------------------------
# Name: check_ip
#
# Description: checks if the IP passed to it is a valid IP address.
#
# Return values:
#   1 = success
#   0 = fail
#   

sub check_ip {
   my ($IP) = @_;
   my ($oct1, $oct2, $oct3, $oct4);

   if ($IP =~ /(\d{1,3})\.(\d{1,3})\.(\d{1,3})\.(\d{1,3})/) {
      $oct1 = $1;
      $oct2 = $2;
      $oct3 = $3;
      $oct4 = $4;

      if (  (($oct1 >= 0) && ($oct1 <= 255))
         && (($oct2 >= 0) && ($oct2 <= 255))
         && (($oct3 >= 0) && ($oct3 <= 255))
         && (($oct4 >= 0) && ($oct4 <= 255)))
      {
         return 1;
      } else {
         warn("$program_name: %s: invalid IP address.\n", $IP);
         return 0;
      }
   } else {
      $IP = quotemeta($IP);
      warn("$program_name: %s: this isn't an IP address.\n", $IP);
      return 0;
   }
}

#-------------------------------------------------------------------------
# Name: check_port
#
# Description: checks if the PORT passed to it is valid.
#
# Return values:
#   1 = success
#   0 = fail
#   

sub check_port {
   my ($PORT) = @_;
   my ($port1);

   if ($PORT =~ /(\d{1,5})/) {
      $port1 = $1;

      if ((($port1 >= 0) && ($port1 <= 65535))) {
         return 1;
      } else {
         warn("$program_name: %s: invalid port number.\n", $PORT);
         return 0;
      }
   } else {
      $PORT = quotemeta($PORT);
      warn("$program_name: %s: this isn't a port number.\n", $PORT);
      return 0;
   }
}

#-------------------------------------------------------------------------
# Name: check_proto
#
# Description: checks if the PROTOCOL passed to it is valid.
#
# Return values:
#   1 = success
#   0 = fail
#   

sub check_proto {
   my ($PROTO) = @_;

   if ($PROTO =~ /tcp/) {
      $proto_show += 1;
      return 1;
   } elsif ($PROTO =~ /udp/) {
      $proto_show += 2;
      return 1;
   } elsif ($PROTO =~ /icmp/) {
      $proto_show += 4;
      return 1;
   } else {
      warn("$program_name: %s: invalid protocol name.\n", $PROTO);
      return 0;
   }
}

#-------------------------------------------------------------------------
# Name: set_list
#
# Description: retrieves the arguments passed to the fuction and
#              return a new list
#

sub set_list {
   my ($check_option) = $_[0];
   shift @_;
   my ($LISTs)      = @_;
   my (@local_list) = ();
   my @aux_list;

   @aux_list = split(/\s*\,\s*/, $LISTs);

   if ($#aux_list != -1) {
      foreach my $list (@aux_list) {
         if ($check_option =~ /IP/) {
            if (&check_ip($list)) {
               push(@local_list, $list);
            }
         } elsif ($check_option =~ /NET/) {
            if (&check_ip($list)) {
               push(@local_list, $list);
            }
         } elsif ($check_option =~ /PORT/) {
            if (&check_port($list)) {
               push(@local_list, $list);
            }
         } elsif ($check_option =~ /PROTO/) {
            if (&check_proto($list)) {
               push(@local_list, $list);
            }
         }
      }
   }
   return @local_list;
}

#-------------------------------------------------------------------------
# Name: check_list
#
# Description: checks if the argument passed to the function belongs to
#              (@*_list)
#
# Return values:
#   1 = success
#   0 = fail
#   

sub check_list {
   my ($ELEMENT) = $_[0];
   shift @_;
   my (@LIST) = @_;

   foreach my $list_elem (@LIST) {
      if ("$ELEMENT" eq "$list_elem") {
         return 1;
      }
   }
   return 0;
}

#-------------------------------------------------------------------------
# Name: check_net_list
#
# Description: checks if the IP passed to the function belongs to
#              (@NET_list)
#
# Return values:
#   1 = success
#   0 = fail
#   

sub check_net_list {
   my ($NET) = @_;

   foreach my $net_list_elem (@NET_list) {
      (my $net_obj = Net::Netmask->new($net_list_elem))
        or die "$program_name: invalid address/netmask.\n";
      if ($net_obj->match($NET)) {
         return 1;
      }
   }
   return 0;
}

#-------------------------------------------------------------------------
# Name: check_filename
#
# Description: checks if filename contains expected characters only.
#
# Return values:
#   file = success
#   0    = fail -- file contains some characters that may lead
#                  to a security problem.
#                  

sub check_filename {
   my ($file) = @_;
   return $1 if ($file =~ /^([\w\-\:\_\.\/]+)$/);
   return 0;
}

#-------------------------------------------------------------------------
# Name: show_usage
#
# Description: print program usage and exit.
#

sub show_usage {
   print <<EOF;

Usage: $program_name [-c honeyd.conf file] [-hV] [-i IP1,IP2,...,IPn] [-n IP1|NET1,...,IPn|NETn] [-p PORT1,PORT2,...,PORTn] [-s NUMBER] [-t PROTO1,PROTO2,...,PROTOn] log-file1 log-file2 ... log-filen
       -c   honeyd.conf file
       -i   list of honeypot's IP addresses.
       -h   display this help and exit.
       -n   source IP or Network addresses for filtering.
       -p   list of destination ports.
       -s   number of records on top (default 10).
       -t   list of protocols.
       -V   display version number and exit.

EOF

   exit 0;
}

#-------------------------------------------------------------------------
# Name: show_version
#
# Description: print program version and exit.

sub show_version {
   printf("$program_name: %s\n", $honeydsum_version);
   exit 0;
}

######################################################################
### honeydsum.pl ends here

